# -*- sh -*-
# At this point, I have to assume that you're already comfortable with the idea that behaviour of
#  characters in the GEDS engine are programmed through a state machine. If you're not, please have
#  a look at worm.cmd first, as it presents the core concepts involved there.

# Funky Funghi is both simpler (less animations and less states) and more sophisticated than 
#  the woodworm. First, Funky Funghi bounces, and is affected by gravity, which suggests a a 
#  movement speed that vary over times. Plus, it doesn't always bounce at the same height, but
#  does a higher jump every 4 bounces, which involves having state about what happened previously.

anim15 8 { # recoil animation, played when FF hits the ground
  spr0 0   # there will be a graphical animation editor ASAP.
  delay 3
  spr0 4
  delay 3
  spr0 0
  delay 3
  spr0 2
  done
}

anim6 8 { # while mid-air the same picture is used continuously.
  spr0 2
  done
}


state0 :anim6 {               # being mid-air, there is gravity affecting us. 
  using gravity(24,1536);     # this is done by attaching a _controller_ to the current state
  test 0 (12,25)-(20,32) 0002 # the GravityController is a C++ class that checks movements are
  test 1 (6,2)-(26,24) 0001   # possible and that increases the vertical speed on every frame.
}

# note that there are two different active collision area. ''test 0'' is a large area that just
# HURTS_PLAYER (0001), just as in woodworm. The second only covers the bottom of funky funghi and 
# has a different flag : STOMPS_PLAYER (0002). That allows some collisions to have a different impact:
# being stomped by a falling funghi can do more damage than just touching the funghi from the side.

state2 :anim15 {
  using stopper
  test 0 (6,2)-(26,32) 0001
}

# to allow speed that varies over time -- but also plenty other features -- we provide every 
# GOB with 16 integer variables that can be manipulated both by controllers and by the transitions
# of the state machine. Those variables are just indexed and do not receive explicit names in
# the script. As an engine-wide convention, v[0] and v[1] contains the horizontal and vertical speed.
# just setting v[0] to 256 will move the the gob at constant speed of 256 pixels per frame.

# variables of a GOB can be freely read through ''v%'' and written '':%'' in the state machine.
# To do so, a stack-based expression evaluation is provided. It allows an expression either to 
# push immediate values on the stack, or to replace some elements at the top of the stack by the
# result of an logic or arithmetic operation. For instance, ''[v4 0 =]'' reads as 
# - read GOB variable #4 and put it on stack
# - put value 0 on stack
# - test those two for equality.
# this is a guardian that ensure the transition can only be taken in specific circumnstances.


state2->state0 on done [v4 0 =] (700 ~ :1)  # conditional transition
state2->state0 on done [t] (500 ~ :1)       # fall-back transition [t] is always true.

# the command part of these transitions ''(500 ~ :1)'' and ''(700 ~ :1)'' alter vertical speed
# (variable 1) by setting a constant value (-500 and -700) respectively. At this point, you might
# like to see some of the 'operations' available in gamescript operations. Think of them as buttons
# on a pocket calculator. Really. Here are the basics :

# +,-,*   2 vals   addition, substraction and multiplication
# /,%     2 vals   integer quotient and remainder
# ~       1 val    opposite-of (that is, ToS := 0 - ToS
# &,|,^   2 vals   AND, OR, XOR bit-wise operations
# !       1 val    binary inverse (ToS := ToS xor FFFF

# =,<,>,<=,>=  2

state0->state2 on fail [t] (v4 1 + 4 % :4 0 :1)

# The ''state0->state2 on fail'' transition is taken everytime Funky Funghi hits the ground.
# it reads variable #4, which hasn't any specific semantic (unlike v[0] and v[1]), and adds 1 module 4 to
# its current value before it writes back the incremented value in variable #4 again.
# That makes v[4] actually counting the number of bounces and reset the value to 0 everytime it reaches 4.

# Sidenote: the expression evaluator system of libgeds works only with signed 16-bit arithmetic

end
